/* -*- mode: c++; c-basic-offset: 4; indent-tabs-mode: nil -*-

this file is part of rcssserver3D
Fri May 9 2003
Copyright (C) 2002,2003 Koblenz University
Copyright (C) 2003 RoboCup Soccer Server 3D Maintenance Group

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <netinet/in.h>

#include <zeitgeist/logserver/logserver.h>

#include "commserver.h"

/*-------------------------------------------------------------------*/
/*!

*/
CommServer::CommServer()
    : M_read_fd( 3 )
    , M_write_fd( 4 )
    , M_read_buf_fd( M_read_fd )
{
    M_buffer_size = DEFAULT_BUFFER_SIZE;
    M_buffer = new char[M_buffer_size];
}

/*-------------------------------------------------------------------*/
/*!

*/
CommServer::~CommServer()
{
    delete [] M_buffer;
}

/*-------------------------------------------------------------------*/
/*!

*/
bool
CommServer::selectInput()
{
    fd_set readfds;
    FD_ZERO( &readfds );
    FD_SET( M_read_fd, &readfds );

    return select( M_read_fd + 1, &readfds, 0, 0, 0 ) > 0;
}

/*-------------------------------------------------------------------*/
/*!

*/
void
CommServer::putOutput( const char* out )
{
    std::strcpy( M_buffer + sizeof( unsigned int ), out );
    unsigned int len = std::strlen( out );
    unsigned int netlen = htonl( len );
    std::memcpy( M_buffer, &netlen, sizeof( netlen ) );
    write( M_write_fd, M_buffer, len + sizeof( netlen ) );
}

/*-------------------------------------------------------------------*/
/*!

*/
bool
CommServer::getInput()
{
    for ( int count = 0; count < 100; ++count )
    {
        int res = M_read_buf_fd.readLengthPrefixed( &M_buffer, &M_buffer_size );
        if ( res < 0 )
        {
            GetLog()->Debug() << "CommServer::GetInput: readLeangthPrefixed returned -1\n";
            return false;
        }
        else if ( res > 0 )
        {
            // successful read
            if ( res == static_cast< int >( M_buffer_size ) )
            {
                GetLog()->Debug() << "CommServer::GetInput: read exactly buffer size! can't null term\n";
                return false;
            }
            M_buffer[res] = 0;
            return true;
        }

        // if we are here, res == 0, which means not enough input

        // this blocks until more input
        if ( ! selectInput() )
        {
            GetLog()->Debug() << "CommServer::GetInput select returned false\n";
            return false;
        }
    }

    GetLog()->Debug() << "CommServer::GetInput: fell out of for loop\n";
    return false;
}
