/* -*- mode: c++; c-basic-offset: 4; indent-tabs-mode: nil -*-

this file is part of rcssserver3D
Fri May 9 2003
Copyright (C) 2002,2003 Koblenz University
Copyright (C) 2003 RoboCup Soccer Server 3D Maintenance Group

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <sstream>

#include "soccer.h"

/*-------------------------------------------------------------------*/
/*!

*/
Soccer::Soccer()
    : Behavior()
    , M_team_name( "unnamed" )
    , M_sensation_count( 0 )
    , M_drive_vec( 0.0f, 0.0f, 0.0f )
{

}

/*-------------------------------------------------------------------*/
/*!

*/
Soccer::~Soccer()
{

}

/*-------------------------------------------------------------------*/
/*!

*/
void
Soccer::doInitAgent()
{
    GetLog()->Debug() << "sending init command\n";

    std::stringstream ss;
    ss << "A(init (unum " << M_team_unum << ") (teamname " << M_team_name << "))";

    M_comm->putOutput( ss.str().c_str() );
}

/*-------------------------------------------------------------------*/
/*!

*/
void
Soccer::doCreateAgent()
{
    // use the create effector to setup the Sensors and Effectors
    GetLog()->Debug() << "creating agent\n";

    M_comm->putOutput( "A(create)" );
}

/*-------------------------------------------------------------------*/
/*!

*/
void
Soccer::doBeam( const salt::Vector3f & position )
{
    GetLog()->Debug()
        << "beaming ["
        << position[0]
        << " " << position[1]
        << " " << position[2]
        << "]\n";

    std::stringstream ss;
    ss << "A(beam "
       << " " << position[0]
       << " " << position[1]
       << "  " << position[2]
       << ")\n";

    M_comm->putOutput( ss.str() );
}

/*
void Soccer::Say(const std::string& msg)
{
    GetLog()->Debug() << "sending init command\n";

    stringstream ss;
    ss << "A(say " << msg << ")";

    M_comm->PutOutput(ss.str().c_str());
}
*/

/*-------------------------------------------------------------------*/
/*!

*/
void
Soccer::doKick( const float angle, const float power )
{
    GetLog()->Debug()
        << "kicking Ball"
        << " angle=" << angle
        << " power=" << power
        << "\n";

    std::stringstream ss;
    ss << "A(kick " << angle << " " << power << ")";

    M_comm->putOutput( ss.str() );
}

/*-------------------------------------------------------------------*/
/*!

*/
void
Soccer::doDrive( const salt::Vector3f& drive_vec )
{
    GetLog()->Debug()
        << "driving ["
        << drive_vec[0] << " "
        << drive_vec[1] << " "
        << drive_vec[2]
        << "]\n";

    if ( drive_vec == getDriveVec() )
    {
        // no explicit change needed
        GetLog()->Debug()
            << "not changing drive vector\n";
        return;
    }

    std::stringstream ss;
    ss << "A(drive"
       << " " << drive_vec[0]
       << " " << drive_vec[1]
       << "  " << drive_vec[2]
       << ")";

    M_comm->putOutput( ss.str() );
}

/*-------------------------------------------------------------------*/
/*!

*/
void
Soccer::think()
{
    GetLog()->Debug()
        << "received sensation " << M_sensation_count << "\n";

    switch ( M_sensation_count ) {
    case 0:
        doCreateAgent();
        break;
    case 1:
    case 2:
    case 4:
    case 5:
        break;
    case 3:
        doInitAgent();
        break;
    default:
        behave();
        break;
    }

    ++M_sensation_count;

    M_comm->putOutput( done_thinking_message() );
}
